DAT programName         byte "TwoEncoderDemo_131214d", 0
CON
{{
  ******* Public Notes *******

  Author: Duane Degn
  Date First Released: December 10, 2013
  Date Latest Released: December 14, 2013

  MIT License

  The current object being demonstated has
  a resolution of 3us (at 80MHz). The main PASM
  loop takes 240 clock cycles to complete.
  This PASM loop time is dependent on encoder
  activity. Active encoder movement will
  likely increase the loop time.

-------------------------------------------------  
  Encoder Information:
  Not modified for two encoders only.  
-------------------------------------------------  
  The algorithm used in the encoder object
  comes out of JonnyMac's Spin Zone article
  "Spinning it Up with Encoders".

  Up to four quadrature encoders may be used
  with the "FourQuadratureMotor" object. The
  "A" and "B" sensor pins need to be
  consecutive I/O pins. Only the first of the
  two pins is listed in the "FIRST_ENCODER_x"
  section of the header object. Provisions
  have been made to reverse the direction
  of the encoder feedback if desired without
  the need to physically swap the wires
  connected to the Propeller. See comments
  within the program about setting the
  encoder direction.

  If an encoder is not used the "FIRST_ENCODER_x"
  pin should be set to -1.

  If an encoder pin is set to a valid I/O pin
  but the I/O pin is left to float, the input
  pins may register phantom transitions.

  There are several constants in the header
  object used to compute speeds.

  The constant "MAX_TRANSITIONS_PER_SEC" is
  used to scale the speed in order to have
  the measured speed have the same range as
  the PWM resolution. For example, if the
  "DEFALUT_PWM_RESOLUTION" constant in the
  child object is set to 1000, then a speed
  setting of 1000 should produce a measured
  speed of 1000. In order for the measured
  speed to match the input speed the constant
  "MAX_TRANSITIONS_PER_SEC" needs to be set
  to the appropriate value.
  
  This appropriate value may be calculated
  based on the motors known max rpm and
  transitions per revolution data or it
  may be directly measured by running this
  program while the motor runs at full
  speed. Currently, the motor will need to
  be powered from another source since this
  program does not yet have a way to input
  speed. 

  Rather than having a single constant for
  the number of transitions per revolution
  of the motor, I used two.
  "TRANS_PER_REV_NUMERATOR" and
  "TRANS_PER_REV_DENOMINATOR".
  This allows for non-integer encoder values.
  For example the Rover 5 has encoders which
  register 1000 transitions every 3 rotations
  of the wheel. Expressing the encoder
  resolution as a fraction allows this 1000/3
  resolution to be accurately stated with
  integers. 

  There are two direction arrays,
  "encoderDirection" and "directionCorrection".
  A pointer to the "encoderDirection" array
  is passed to the encoder object which
  populates the array with ones or negative
  ones based on the sensed direction the
  encoders are turning. The other array
  "directionCorrection" is filled with
  constants from the header object. These
  constants are used to correct the direction
  reported by the encoders so the wires
  of the encoders do not need to be physically
  swapped.
  
  
-------------------------------------------------  
  Features To Add:
-------------------------------------------------  
  Be able to input target speeds.

  Think about 64-bit math.
 
}}
{
  ******* Private Notes *******

  Rename "TwoEncoderDemo_131214a" from "EncoderDemo_131214a".
  14a Appears to have trouble counting transitions.
  14b Fixed text overwriting digits problem. Seems to work.
  14b Appears usable.
  
}
CON                     
                     
  _clkmode = xtal1 + pll16x 
  _xinfreq = 5_000_000

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MICROSECOND = CLK_FREQ / 1_000_000
  MILLISECOND = CLK_FREQ / 1_000
  
  TEMP_MESSAGE_DISPLAY_TIME = CLK_FREQ * 4
  
  MAX_POSSIBLE_ENCODERS = Encoders#TOTAL_ENCODERS
  ARRAYS_TO_FREEZE = 3
  LONGS_TO_FREEZE = MAX_POSSIBLE_ENCODERS * ARRAYS_TO_FREEZE
  MAX_POSITIVE = $7FFFFFFF
  LARGEST_PRACTICAL_MULTIPLIER = MAX_POSITIVE 
  LARGEST_POWER_OF_TEN_LIKELY = 10000

  HEADING_ROWS = 5
 ' MENU_2ND_COL_LOC_X = 54
 ' MENU_LOC_Y = HEADING_ROWS
 ' MENU_ROWS = 0 '11 
 ' ACTIVE_PARAMETER_LOC_Y = MENU_LOC_Y + MENU_ROWS
 ' ACTIVE_PARAMETER_ROWS = 3
  LOOP_TIMES_LOC_Y = HEADING_ROWS 'ACTIVE_PARAMETER_LOC_Y + ACTIVE_PARAMETER_ROWS
  LOOP_TIMES_ROWS = 5
  ENCODER_DATA_LOC_Y = LOOP_TIMES_LOC_Y + LOOP_TIMES_ROWS 
  ENCODER_DATUM_ROWS = 5
  ENCODER_DATA_ROWS = ENCODER_DATUM_ROWS * Encoders#TOTAL_ENCODERS
  IO_PIN_STATES_LOC_Y = ENCODER_DATA_LOC_Y + ENCODER_DATA_ROWS + 1
  IO_PIN_STATES__ROWS = 4
'  PID_DATA_LOC_Y = IO_PIN_STATES_LOC_Y
  'IO_PIN_STATES__ROWS = 5
'  TARGET_SPEED_LOC_Y = IO_PIN_STATES_LOC_Y + IO_PIN_STATES__ROWS
 ' TARGET_SPEED_ROWS = 5
  SPEED_MESSAGE_LOC_Y = IO_PIN_STATES_LOC_Y + IO_PIN_STATES__ROWS 'TARGET_SPEED_LOC_Y + TARGET_SPEED_ROWS 
  SPEED_MESSAGE_ROWS = 4
  TEMP_MESSAGE_LOC_Y = SPEED_MESSAGE_LOC_Y + SPEED_MESSAGE_ROWS
  TEMP_MESSAGE_ROWS = 3
  
  QUOTE = 34  ' double quotes
  BELL = 7
  
  LINE_SIZE = 40
  ASTRIX_IN_ROW = LINE_SIZE
  ' activeParameter enumeration
  #0, PROPORTIONAL_PARAMETER, INTEGRAL_PARAMETER, DERIVATIVE_PARAMETER, {
    } MAX_INTEGRAL_PARAMETER, PID_DENOMINATOR_PARAMETER, SMALL_INCREMENT_SIZE_PARAMETER, {
    } LARGE_INCREMENT_SIZE_PARAMETER, TOP_SPEED_FEEDBACK_PARAMETER, ACCELERATION_PARAMETER, {
      DELTA_TIME_PARAMETER,
    } FREQUENCY_PARAMETER, RESOLUTION_PARAMETER

  TOTAL_PARENT_PARAMETERS = ACCELERATION_PARAMETER + 1 'TOP_SPEED_FEEDBACK + 1

  #0, ERROR_MESSAGE, SPEED_MESSAGE, ALTERNATE_MESSAGE
  
  MAX_INDEX_TEMP_MESSAGE_TYPES = ALTERNATE_MESSAGE 
  TEMP_MESSAGE_TYPES = MAX_INDEX_TEMP_MESSAGE_TYPES + 1

  CONTROL_LOOP_STACK = 64

  MAX_MESSAGE_SIZE = 100

  MAX_SCALED_SPEED = 1000
  
VAR

  long controlStack[CONTROL_LOOP_STACK], controlCog
  long measuredStack, previousMeasuredStack
  long controlLoopCounter, previousCounter
  long watchdogTimer, watchdogInterval
  long directionCorrection[MAX_POSSIBLE_ENCODERS], output[MAX_POSSIBLE_ENCODERS]
  long targetSpeed[MAX_POSSIBLE_ENCODERS], pseudoRampedSpeed[MAX_POSSIBLE_ENCODERS]
  long pseudoError[MAX_POSSIBLE_ENCODERS], previousError[MAX_POSSIBLE_ENCODERS]
  long proportional[MAX_POSSIBLE_ENCODERS]
  long integralSum[MAX_POSSIBLE_ENCODERS], derivative[MAX_POSSIBLE_ENCODERS]
  long zeroSpeedTime
  long newCount[MAX_POSSIBLE_ENCODERS], pulseToSpeedNumerator
  long transPerSecSpeed[MAX_POSSIBLE_ENCODERS], scaledSpeed[MAX_POSSIBLE_ENCODERS] 
  long rpmSpeed[MAX_POSSIBLE_ENCODERS]', linearSpeed[MAX_POSSIBLE_ENCODERS], linearUnitsPtr 
  long integerMathEnhancer[2], otherPortionOfEnhancer[2], powerOfTenPortionOfEnhancer[2]
  long powerOfTenPortionsOfEnhancer, otherPortionsOfEnhancer
  long parentParameter[TOTAL_PARENT_PARAMETERS], tempMessageTimer[TEMP_MESSAGE_TYPES]
  long smallIncrementSize[TOTAL_PARENT_PARAMETERS], largeIncrementSize[TOTAL_PARENT_PARAMETERS]
  long controlAlgorithmRate, maxPossibleRefresh, controlAlgorithmInterval
  long debugInterval, negPidMaxIntegral, previousParameterValue
  long pseudoGlobalAccel[MAX_POSSIBLE_ENCODERS] 
  
  '' The following longs need to be kept in order and together.
  long frozenTransitionTime[MAX_POSSIBLE_ENCODERS], frozenLastPulse[MAX_POSSIBLE_ENCODERS]
  long frozenEncoderCount[MAX_POSSIBLE_ENCODERS]
  long previousTransitionTime[MAX_POSSIBLE_ENCODERS], previousLastPulse[MAX_POSSIBLE_ENCODERS]
  long previousEncoderCount[MAX_POSSIBLE_ENCODERS]
  long legacyLongsNotUsed[MAX_POSSIBLE_ENCODERS]
  long transitionTime[MAX_POSSIBLE_ENCODERS], timeOfLastPulse[MAX_POSSIBLE_ENCODERS]
  long totalEncoderCount[MAX_POSSIBLE_ENCODERS]
  long encoderDirection[MAX_POSSIBLE_ENCODERS] 
  '' The above longs need to be kept in order and together.

  byte activeChannelFlag[MAX_POSSIBLE_ENCODERS], stillStoppedFlag[MAX_POSSIBLE_ENCODERS]
  byte activeParameter, previousActiveParameter
  byte tempMessageFlag[TEMP_MESSAGE_TYPES], tempMessageLoc[2 * TEMP_MESSAGE_TYPES]
  byte message0[MAX_MESSAGE_SIZE], message1[MAX_MESSAGE_SIZE], message2[MAX_MESSAGE_SIZE]
  
OBJ

  Pst : "Parallax Serial Terminal"                      ' uses one cog
  Encoders : "TwoEncoders131214a"                       ' uses one cog
  Header : "HeaderEncodersOnly131214a"
  Format : "StrFmt"
  
PUB Main 
'' This method starts in cog #0
'' This method starts two additional cogs (encoder driver and serial driver).
'' Presently there are five unused cogs.

  directionCorrection[0] := Header#FORWARD0  ' Note about direction variables
  directionCorrection[1] := Header#FORWARD1  ' in comments under "Encoder Information."

  '' "directionCorrection" allows direction reversal without requiring wires be physically moved.
  '' If the encoder direction is reversed from direction desired, change the "FORWARDx" to -1 
  '' in header object.

  pulseToSpeedNumerator := (clkfreq / Header#MAX_TRANSITIONS_PER_SEC) * MAX_SCALED_SPEED
  '' pulseToSpeedNumerator used to scale speed in attempt to get target speed and measured speed
  '' to match

  ComputerMathEnhancers
    
  zeroSpeedTime := pulseToSpeedNumerator + (pulseToSpeedNumerator / 2)
  '' How long a pulse needs to be to be considered stopped.
  
  Pst.Start(115200)
  
  Encoders.StartEncoders(Header.GetEncoderPins, @totalEncoderCount, @transitionTime, @encoderDirection)
 
  'waitcnt(clkfreq * 2 + cnt)

  RefreshStaticScreen
  
  DebugLoop
 
PUB DebugLoop | frozenCnt, frozenIna, combinedDirection, newTime, oldTime 

  repeat
    newTime := cnt
    debugInterval := newTime - oldTime
    oldTime := newTime


    repeat result from 0 to MAX_INDEX_TEMP_MESSAGE_TYPES
      if tempMessageFlag[result]
        if cnt - tempMessageTimer[result] > TEMP_MESSAGE_DISPLAY_TIME
          tempMessageFlag[result] := 0
          ClearLines(tempMessageLoc[2 * result], tempMessageLoc[(2 * result) + 1])
  
    result := Pst.RxCount
    if result
      ReadInput
    
    DisplayLoopTimes
       
    longmove(@previousTransitionTime, @frozenTransitionTime, LONGS_TO_FREEZE)
    longmove(@frozenTransitionTime, @transitionTime, LONGS_TO_FREEZE)
    '' Encoder data is copied to "frozen" arrays so the values don't change
    '' while being compared. This insures the "previous" value stored is
    '' the same value used in the comparison.
    
    frozenCnt := cnt

    CheckForStoppedChannels    
    'DisplayEnhancers
       
    repeat result from 0 to Encoders#MAX_ENCODER_INDEX
      combinedDirection := encoderDirection[result]  * directionCorrection[result]
      
      if frozenCnt - frozenLastPulse[result] < zeroSpeedTime and stillStoppedFlag[result] == 0
        transPerSecSpeed[result] := ComputeTransPerSec(frozenTransitionTime[result], combinedDirection)
        rpmSpeed[result] := ComputeRpm(frozenTransitionTime[result], combinedDirection)
        scaledSpeed[result] := ComputeScaledSpeed(frozenTransitionTime[result], combinedDirection)
      else ' slower than allowed
        transPerSecSpeed[result] := 0
        rpmSpeed[result] := 0
        scaledSpeed[result] := 0
      ifnot activeChannelFlag[result]  ' Channel not currently active
        if timeOfLastPulse[result]  ' if pulse has been written by encoder driver set channel active
          activeChannelFlag[result] := 1
       ' else
       '   next  ' Channel still not active skip display portion of loop
      DisplayEncoderChannel(result)
      
      DividingLine      
    DisplayIo
    
    'waitcnt(clkfreq / 10 + cnt) 

PUB ReadInput : localCharacter

  localCharacter := Pst.CharIn
  
  case localCharacter
   
    "c", "C":
      RefreshStaticScreen
    
    other:
      DisplayError(@characterInputError)
                     
PUB DisplayLoopTimes

  Pst.Position(0, LOOP_TIMES_LOC_Y)
  Pst.Str(string(" Encoder loop time = "))
  result := EncoderS.GetDebugTime
  Pst.Dec(result)
  Pst.Str(string(" clock cycles or ")) 
  Pst.Dec(result / MICROSECOND)
  Pst.Str(string(" us")) 
  Pst.ClearEnd
  Pst.Newline
 
  
  Pst.Str(string("  The debug loop takes "))
  Pst.Dec(debugInterval)
  Pst.Str(string(" clock cycles or ")) 
  Pst.Dec(debugInterval / MILLISECOND )
  Pst.Str(string(" ms to complete."))
  Pst.ClearEnd
  DividingLine

PUB DisplayIo | frozenIna

  Pst.Position(0, IO_PIN_STATES_LOC_Y)
  Pst.str(string("             I/O Pin Numbers"))
  Pst.ClearEnd
  Pst.str(string(13, "      tens:   3           2            1"))
  Pst.ClearEnd
  Pst.str(string(13, "     units:  1098_7654_3210_9876_5432_1098_7654_3210"))
  Pst.ClearEnd
  frozenIna := ina
  Pst.str(string(13, " ina state: "))
  ReadableBin(frozenIna)
  Pst.ClearEnd
  DividingLine
    
PUB DisplayEncoderChannel(channelIndex)

  {newCount[channelIndex] := frozenEncoderCount[channelIndex] - previousEncoderCount[channelIndex] 
  Pst.NewLine
  Pst.ClearEnd
  Pst.Str(string(13, "       newCount["))
  Pst.Dec(channelIndex)
  Pst.Str(string("] = "))
  Pst.Dec(newCount[channelIndex])
  Pst.ClearEnd   }
  Pst.Position(0, ENCODER_DATA_LOC_Y + (channelIndex * ENCODER_DATUM_ROWS))
  Pst.Str(string("     totalCount["))
  Pst.Dec(channelIndex)
  Pst.Str(string("] = "))
  'Pst.Dec(frozenEncoderCount[channelIndex])
  'Pst.ClearEnd
  Pad(frozenEncoderCount[channelIndex], 10)
  Pst.PositionX(33)
  Pst.Str(string(", transitionTime["))
  Pst.Dec(channelIndex)
  Pst.Str(string("] = "))
  'Pst.Dec(frozenTransitionTime[channelIndex])
  'Pst.Str(string(", = "))
  Pad(frozenTransitionTime[channelIndex], 8)
  Pst.PositionX(64)
  Pst.Str(string(" clock cycles, average of "))
  Pst.Dec(Encoders#BUFFER_SIZE_ENCODER_TIME)
  Pst.Str(string(" readings"))
  Pst.ClearEnd
  Pst.Str(string(13, "          speed["))
  Pst.Dec(channelIndex)
  Pst.Str(string("] = "))
  'Pst.Dec(transPerSecSpeed[channelIndex])
  Pad(transPerSecSpeed[channelIndex], 10) ' use ten digit for formatting purposes
  Pst.PositionX(33)
  'Pst.ClearEnd 
  Pst.Str(string("(transitions per second)"))
  Pst.ClearEnd
  'Pst.NewLine 
  'Pst.ClearEnd 
  Pst.Str(string(13, "    scaledSpeed["))
  Pst.Dec(channelIndex)
  Pst.Str(string("] = "))
  'Pst.Dec(scaledSpeed[channelIndex])
  'Pst.ClearEnd
  Pad(scaledSpeed[channelIndex], 10)
  Pst.PositionX(33)         
  Pst.Str(string(" Scaled so max transitions per sec ("))
  Pst.Dec(Header#MAX_TRANSITIONS_PER_SEC)
  Pst.Str(string(") equals full speed of "))
  Pst.Dec(MAX_SCALED_SPEED) 
  Pst.Str(string(" (MAX_SCALED_SPEED)."))
  Pst.ClearEnd
  'Pst.NewLine
  'Pst.ClearEnd
  Pst.Str(string(13, "              rpm =        "))
  DecPoint(rpmSpeed[channelIndex], powerOfTenPortionsOfEnhancer)
  Pst.Str(string("   "))
  'Pst.ClearEnd
  Pst.Str(string(" Based on "))
  Pst.Dec(Header#TRANS_PER_REV_NUMERATOR) 
  if Header#TRANS_PER_REV_DENOMINATOR <> 1
    Pst.Str(string(" / "))
    Pst.Dec(Header#TRANS_PER_REV_DENOMINATOR)
  Pst.Str(string(" transitions per revolution."))
  
  Pst.ClearEnd
  'DividingLine
  
PUB RefreshStaticScreen

  Pst.Clear
  DisplayHeading
   
PUB DisplayHeading

  Pst.Home
  Pst.ClearEnd
  Pst.Str(string(13, "        name of program = "))
  Pst.Str(@programName)
  Pst.ClearEnd
  Pst.Str(string(13, " name of encoder object = "))
  Pst.Str(Encoders.GetObjectName)
  Pst.ClearEnd
  Pst.Str(string(13, "  name of header object = "))
  Pst.Str(Header.GetObjectName)
  Pst.ClearEnd
  'Pst.NewLine
  DividingLine
  'Pst.ClearEnd

PUB FindString(startPtr, stringIndex)

  result := startPtr
  repeat while stringIndex--
    repeat while byte[result++]

PUB DisplayError(errorPtr) | fillSpaces[2]

  Pst.Char(BELL)
  result := strsize(errorPtr)
  fillSpaces[0] := ((ASTRIX_IN_ROW - result) - 4) / 2
  fillSpaces[1] := fillSpaces[0] + (((ASTRIX_IN_ROW - result) - 4) // 2)
  
  result := @message1
  bytefill(result, " ", ASTRIX_IN_ROW)
  result := Format.str(result, @sideErrorBox)
  result += fillSpaces[0]
  result := Format.str(result, errorPtr)
  result += fillSpaces[1]
  result := Format.str(result, @sideErrorBox)
  byte[result] := 0

  DisplayMessage(ERROR_MESSAGE, @astrixRow, @message1, @astrixRow)
  
  Pst.Char(BELL)
  
PUB DisplayMessage(messageType, messagePtr0, messagePtr1, messagePtr2) | fillSpaces
 
  if messageType == SPEED_MESSAGE
    result := SPEED_MESSAGE_LOC_Y
  else
    result := TEMP_MESSAGE_LOC_Y
  Pst.Position(0, result)
  if messagePtr0
    Pst.Str(messagePtr0)
  Pst.ClearEnd
  Pst.NewLine
  if messagePtr1
    Pst.Str(messagePtr1)
  Pst.ClearEnd
  Pst.NewLine
  if messagePtr2
    Pst.Str(messagePtr2)
  Pst.ClearEnd
 
  tempMessageTimer[messageType] := cnt
  tempMessageFlag[messageType] := 1 

  tempMessageLoc[2 * messageType] := result
  tempMessageLoc[(2 * messageType) + 1] := result + 2
  
PUB DividingLine

  Pst.NewLine
  Pst.Str(@dashRow) 
  Pst.ClearEnd
  Pst.NewLine
  
PUB CheckForStoppedChannels
'' Measuring speed from pulse lengths
'' become meaningless when an encoder
'' has stopped moving.

  repeat result from 0 to 3
    ifnot activeChannelFlag[result]
      next
    'Pst.Str(string("  , ["))
    'Pst.Dec(result)
    'Pst.Str(string("] "))
    stillStoppedFlag[result] := 0
    if scaledSpeed[result] == 0
      if timeOfLastPulse[result] == previousLastPulse[result]
        'Pst.Str(string("still zero"))
        'frozenLastPulse[result] :=  (frozenCnt - zeroSpeedTime) '- 1
        ' I'm not sure if the stopped encoder is handled correctly.
        stillStoppedFlag[result] := 1
      else
        'Pst.Str(string("  new zero"))
    else
      'Pst.Str(string("  not zero"))    
  'Pst.ClearEnd

PUB ComputeTransPerSec(time, direction)

  result := clkfreq / time * direction

PUB ComputeRpm(time, direction)

  result := (((((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0] * {
          } direction) / time) * 60) * integerMathEnhancer[1]) / {
          } Header#TRANS_PER_REV_NUMERATOR) / otherPortionsOfEnhancer
       
PUB ComputeScaledSpeed(time, direction)

  result := (pulseToSpeedNumerator / frozenTransitionTime[result]) * direction

PUB ComputerMathEnhancers
'' This method is used to find variables to optimize the precision
'' of the integer math.
'' These values need to be computed for each set of encoders 
'' based on the encoder resolution and speed of the motor.
'' These variables are not required but do improve the accuracy
'' of the computed speeds.
'' I plan to investigate using 64-bit math in order
'' to further improve the accuracy of the speed calculations.

  integerMathEnhancer[0] := (LARGEST_PRACTICAL_MULTIPLIER / clkfreq) / Header#TRANS_PER_REV_DENOMINATOR
  '' The pulse number used in the speed equations can result in significant rounding
  '' errors do to integer math. This factor is used to reduce the rounding error by
  '' increasing the original value to near the point of 32-bit(signed) rollover.
  
  powerOfTenPortionOfEnhancer[0] := LARGEST_POWER_OF_TEN_LIKELY
  repeat
    otherPortionOfEnhancer[0] := integerMathEnhancer[0] / powerOfTenPortionOfEnhancer[0]
    if otherPortionOfEnhancer[0] == 0
      powerOfTenPortionOfEnhancer[0] /= 10
  while otherPortionOfEnhancer[0] == 0 and powerOfTenPortionOfEnhancer[0] > 1
  if otherPortionOfEnhancer[0] == 0
    otherPortionOfEnhancer[0] := 1

  integerMathEnhancer[0] := otherPortionOfEnhancer[0] * powerOfTenPortionOfEnhancer[0]

  result := clkfreq / ((Header#MAX_TRANSITIONS_PER_SEC * 3) / 2) ' shortest possible transition time
  integerMathEnhancer[1] := result / 60 
  '' This factor is used to reduce the rounding error by
  '' increasing the intermediate value to near the point of 32-bit(signed) rollover.
  
  powerOfTenPortionOfEnhancer[1] := LARGEST_POWER_OF_TEN_LIKELY
  repeat
    otherPortionOfEnhancer[1] := integerMathEnhancer[1] / powerOfTenPortionOfEnhancer[1]
    if otherPortionOfEnhancer[1] == 0
      powerOfTenPortionOfEnhancer[1] /= 10
  while otherPortionOfEnhancer[1] == 0 and powerOfTenPortionOfEnhancer[0] > 1
  if otherPortionOfEnhancer[1] == 0
    otherPortionOfEnhancer[1] := 1
  
  integerMathEnhancer[1] := otherPortionOfEnhancer[1] * powerOfTenPortionOfEnhancer[1]

  powerOfTenPortionsOfEnhancer := powerOfTenPortionOfEnhancer[0] * powerOfTenPortionOfEnhancer[1]
  otherPortionsOfEnhancer := otherPortionOfEnhancer[0] * otherPortionOfEnhancer[1]
         
PUB DisplayEnhancers

  Pst.Str(string(13, " LARGEST_PRACTICAL_MULTIPLIER = "))
  Pst.Dec(LARGEST_PRACTICAL_MULTIPLIER)
  Pst.ClearEnd
  Pst.Str(string(13, " integerMathEnhancer[0] = "))
  Pst.Dec(integerMathEnhancer[0])
  Pst.ClearEnd
  Pst.Str(string(13, " otherPortionOfEnhancer[0] = "))
  Pst.Dec(otherPortionOfEnhancer[0])
  Pst.ClearEnd
  Pst.Str(string(13, " powerOfTenPortionOfEnhancer[0] = "))
  Pst.Dec(powerOfTenPortionOfEnhancer[0])
  Pst.ClearEnd
  Pst.Str(string(13, " integerMathEnhancer[1] = "))
  Pst.Dec(integerMathEnhancer[1])
  Pst.ClearEnd
  Pst.Str(string(13, " otherPortionOfEnhancer[1] = "))
  Pst.Dec(otherPortionOfEnhancer[1])
  Pst.ClearEnd
  Pst.Str(string(13, " powerOfTenPortionOfEnhancer[1] = "))
  Pst.Dec(powerOfTenPortionOfEnhancer[1])
  Pst.ClearEnd
  Pst.Str(string(13, " powerOfTenPortionsOfEnhancer = "))
  Pst.Dec(powerOfTenPortionsOfEnhancer)
  Pst.ClearEnd
  Pst.Str(string(13, " otherPortionsOfEnhancer = "))
  Pst.Dec(otherPortionsOfEnhancer)
  Pst.ClearEnd

PUB ClearLines(lineToClear, lastLine)

  repeat
    Pst.Position(0, lineToClear++)
    Pst.ClearEnd 
  while lineToClear =< lastLine
  
PUB ReadableBin(localValue)

  Pst.char("%")
  Pst.bin(localValue >> 28, 4)
  Pst.char("_")
  Pst.bin(localValue >> 24, 4)
  Pst.char("_")
  Pst.bin(localValue >> 20, 4)
  Pst.char("_")
  Pst.bin(localValue >> 16, 4)
  Pst.char("_")
  Pst.bin(localValue >> 12, 4)
  Pst.char("_")
  Pst.bin(localValue >> 8, 4)
  Pst.char("_")
  Pst.bin(localValue >> 4, 4)
  Pst.char("_")
  Pst.bin(localValue, 4)

PUB DecPoint(value, denominator)
  if value < 0
    Pst.Char("-")
    -value
      
  if value => denominator
    result := value / denominator
    Pst.Dec(result)
    value //= denominator     
  else    
    Pst.Char("0")
  Pst.Char(".")  
  repeat while denominator > 1
    denominator /= 10
    if value => denominator
      result := value / denominator
      Pst.Dec(result)
      value //= denominator
    else
      Pst.Char("0")
      
PUB Pad(value, digits) | negFlag
'' This method will pad a number with spaces.
'' The number will take up digits + 1 positions.
'' The extra position is used for a negative
'' sign in case of a negative number.

  if value < 0
    negFlag := 1
    -value
  else
    negFlag := 0
    
  result := 1
  repeat digits - 1
    result *= 10
    
  repeat digits
    if value < result 
      Pst.Char(" ")
      result /= 10
    else
      if negFlag
        Pst.Char("-")
      else
        Pst.Char(" ")
      Pst.Dec(value)
      quit

  ifnot result
    Pst.Char("0")
           
PUB JoinStr(destinationPtr, strPtr0, strPtr1)

  result := strsize(strPtr0)
  bytemove(destinationPtr, strPtr0, result)
  destinationPtr += result
  result := strsize(strPtr1)
  bytemove(destinationPtr, strPtr1, result)
  
  result += destinationPtr 
  byte[result] := 0  ' terminate the string
  
PUB StepByStepDebug(localIndex)
'' Used to catch 32-bit overflow.

  Pst.Str(string(13, "clkfreq * Header#TRANS_PER_REV_DENOMINATOR = "))
  Pst.Dec(clkfreq * Header#TRANS_PER_REV_DENOMINATOR)
  Pst.ClearEnd
  Pst.Str(string(13, "clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0] = "))
  Pst.Dec(clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0])
  Pst.ClearEnd
  Pst.Str(string(13, "(clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0]) "))
  Pst.Str(string("/ frozenTransitionTime[result] = "))
  Pst.Dec((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0]) / frozenTransitionTime[localIndex])
  Pst.ClearEnd
  Pst.Str(string(13, "((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0]) "))
  Pst.Str(string("/ frozenTransitionTime[result]) * 60 = "))
  Pst.Dec(((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0]) / frozenTransitionTime[localIndex]) * {
      } 60)
  Pst.ClearEnd
  Pst.Str(string(13, "(((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0]) "))
  Pst.Str(string("/ frozenTransitionTime[result]) * 60) * integerMathEnhancer[1] = "))
  Pst.Dec((((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0]) / frozenTransitionTime[localIndex]) * {
      } 60) * integerMathEnhancer[1])
  Pst.ClearEnd
  Pst.Str(string(13, "(((((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0]) "))
  Pst.Str(string("/ frozenTransitionTime[result]) * 60) * integerMathEnhancer[1]) / "))
  Pst.Str(string("Header#TRANS_PER_REV_NUMERATOR) / otherPortionsOfEnhancer = "))
  Pst.Dec((((((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0]) / frozenTransitionTime[localIndex]) * {
      } 60) * integerMathEnhancer[1]) / Header#TRANS_PER_REV_NUMERATOR) / otherPortionsOfEnhancer)
  Pst.ClearEnd

PRI ExtraDebug
'' Not used.

  {Pst.Str(string(13, "    linearSpeed["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(linearSpeed[result])
  Pst.ClearEnd
  Pst.Str(string(13, " based on wheel diameter of "))
  Pst.Dec(Header#DIAMETER_OF_WHEEL) 
  Pst.Str(Header.GetLinearUnits)
  Pst.ClearEnd      }
   
  Pst.Str(string(13, "previousTransitionTime["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(previousTransitionTime[result])
  Pst.ClearEnd
  Pst.Str(string(13, "previousLastPulse["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(previousLastPulse[result])
  Pst.ClearEnd
  Pst.Str(string(13, "previousEncoderCount["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(previousEncoderCount[result])
  Pst.ClearEnd
  Pst.Str(string(13, "frozenTransitionTime["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(frozenTransitionTime[result])
  Pst.ClearEnd
  Pst.Str(string(13, "frozenLastPulse["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(frozenLastPulse[result])
  Pst.ClearEnd
  Pst.Str(string(13, "previousEncoderCount["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(frozenEncoderCount[result])
  Pst.ClearEnd
  Pst.Str(string(13, "legacyLongsNotUsed["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(legacyLongsNotUsed[result])
  Pst.ClearEnd
  Pst.Str(string(13, "transitionTime["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(transitionTime[result])
  Pst.ClearEnd
  Pst.Str(string(13, "timeOfLastPulse["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(timeOfLastPulse[result])
  Pst.ClearEnd
  Pst.Str(string(13, "totalEncoderCount["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(totalEncoderCount[result])
  Pst.ClearEnd

DAT

astrixRow               byte "*"[ASTRIX_IN_ROW], 0
dashRow                 byte "-"[LINE_SIZE], 0
sideErrorBox            byte "**", 0

characterInputError     byte "Not a Valid Input Character", 0
parameterNotYetError    byte "Parameter Not Presently Adjustable", 0  ' <- Barely fits in error box.
parameterError          byte "AP Not Adjustable This Way", 0  ' 
speedInputError         byte "Not a Valid Speed", 0
refreshError            byte "Refresh Rate Too Fast", 0
                        