DAT objectName          byte "TwoEncoders131214a", 0
CON
{{
  ******* Public Notes *******

  This program is used to monitor two quadrature
  encoders like those used by the Rover 5 chassis.
  See comments at start of each method.

  This object was written by Duane Degn

  The code based on a the encoder technique shown in a
  Nuts & Volts article by Jon "JonnyMac" McPhalen.
  
}}
{
  ******* Private Notes *******

  Rename "TwoEncoders131214a" from "FourQuadratureMotors131213b".
  131214a Start removing PWM and two of the encoders code.
  
}
CON

  TOTAL_ENCODERS = 2            '' Not changeable
                                                               
  MAX_ENCODER_INDEX = TOTAL_ENCODERS - 1                              
  'SPEED_BUFFER_SIZE = 16
  

  BITS_TO_AVERAGE_ENCODER_TIME = 4   '' user changeable
 
  BUFFER_SIZE_ENCODER_TIME = 1 << BITS_TO_AVERAGE_ENCODER_TIME
 
  MAX_INDEX_ENCODER_TIME = BUFFER_SIZE_ENCODER_TIME - 1

     
VAR

  long cog
  long debugTime
  long pulseBuffer[BUFFER_SIZE_ENCODER_TIME * TOTAL_ENCODERS]
  long largestMotorsInUseIndex
             
PUB StartEncoders(encoderPinPtr, encoderPtr, speedPtr, directionFlagPtr_) | localIndex
'' This method expects encoder pins to be stored sequentially
'' in an array of bytes at encoderPinPtr.
'' The value "encoderPtr" should be the memory location
'' of four sequential longs where incremented encoder values
'' will be written.
'' The value "speedPtr" should be the memory location
'' of four sequential longs where the sum of "speedBuffer" values
'' will be written.
'' This method needs to be called to start using object.

  directionFlagPtr0 := directionFlagPtr_
  directionFlagPtr1 := directionFlagPtr0 + 4

  
  encoderPin0 := byte[encoderPinPtr] 
  encoderPin1 := byte[encoderPinPtr][1] 

  timeAddress := @debugTime

  headRollover := BUFFER_SIZE_ENCODER_TIME * 4

  pulsePtr0 := @pulseBuffer
  pulsePtr1 := pulsePtr0 + (headRollover)

  pulseBufferSumPtr0 := speedPtr 
  pulseBufferSumPtr1 := pulseBufferSumPtr0 + 4
  lastUpdatedPtr0 := pulseBufferSumPtr1 + 4 
  lastUpdatedPtr1 := lastUpdatedPtr0 + 4      
 
  Stop
  
  result := cog := cognew(@enter, encoderPtr) + 1

PUB Stop
'' frees a cog

  if cog
    cogstop(cog~ - 1)

PUB GetObjectName
'' This method returns the name of this object.
'' This method is not needed for proper operation of object
'' and may be deleted.

  result := @objectName

PUB GetDebugTime
'' This method returns the value of "debugTime" to aid
'' in debugging program.
'' This method is not needed for proper operation of object
'' and may be deleted.

  result := debugTime
  
PUB GetBufferLocation
'' This method returns the location of "speedBuffer" to aid
'' in debugging program.
'' This method is not needed for proper operation of object
'' and may be deleted.

  result := @pulseBuffer
  
DAT                     org       0

enter                   mov     encoderAddress, par 
                        mov     encoderAddress0, encoderAddress 
                        mov     encoderAddress1, encoderAddress0
                        add     encoderAddress1, #4 
                                            
                        mov     oldscan0, ina                   ' get start-up inputs
                        mov     oldscan1, oldscan0                                         
                        shr     oldscan0, encoderPin0           ' z-align scan   
                        and     oldscan0, #%11                  ' clear other bits
                        shr     oldscan1, encoderPin1           ' z-align scan   
                        and     oldscan1, #%11                  ' clear other bits

'-------------------------------------------------------------------------------

encloop                 mov     previousTime, loopTimer   ' first reading will be garbage
                        mov     loopTimer, cnt            ' find time to execute loop
                        mov     deltaTime, loopTimer
                        sub     deltaTime, previousTime
                                   
                        wrlong  deltaTime, timeAddress     ' Used to keep track of loop timing        
                                                           ' The loop could be sped up if timing
                                                           ' weren't measured.
                                                           
                        mov     originalScan, ina               ' get current inputs

                        mov     newscan, originalScan           
                        shr     newscan, encoderPin0
                        mov     oldScan, oldscan0               
                        mov     encoderTotal, encoderTotal0     
                        mov     encoderAddress, encoderAddress0
                                
                        mov     lastPulse, lastPulse0
                        mov     pulseBufferSumPtr, pulseBufferSumPtr0  ' no change
                        mov     pulseHead, pulseHead0   ' will change
                        mov     lastUpdatedPtr, lastUpdatedPtr0    ' no change
                        mov     pulseTotal, pulseTotal0   ' will change
                        mov     pulsePtr, pulsePtr0       ' no change
                        mov     directionFlagPtr, directionFlagPtr0

                        call    #chkEncoder                     ' run algorithm with active variables
                        mov     oldscan0, newscan 
                        mov     encoderTotal0, encoderTotal
                        add     encoderQuanta0, encoderChange

                        '**
                        mov     pulseTotal0, pulseTotal
                        mov     pulseHead0, pulseHead
                        mov     lastPulse0, lastPulse
                                                                                                
                        mov     newscan, originalScan           
                        shr     newscan, encoderPin1
                        mov     oldScan, oldscan1               
                        mov     encoderTotal, encoderTotal1     
                        mov     encoderAddress, encoderAddress1   

                        mov     lastPulse, lastPulse1
                        mov     pulseBufferSumPtr, pulseBufferSumPtr1  ' no change
                        mov     pulseHead, pulseHead1   ' will change
                        mov     lastUpdatedPtr, lastUpdatedPtr1    ' no change
                        mov     pulseTotal, pulseTotal1   ' will change
                        mov     pulsePtr, pulsePtr1       ' no change
                        mov     directionFlagPtr, directionFlagPtr1
                        
                        call    #chkEncoder               ' run algorithm with active variables
                        mov     oldscan1, newscan 
                        mov     encoderTotal1, encoderTotal
                        add     encoderQuanta1, encoderChange

                        mov     pulseTotal1, pulseTotal
                        mov     pulseHead1, pulseHead
                        mov     lastPulse1, lastPulse

                       
                        jmp     #encloop

'-------------------------------------------------------------------------------
                        
chkencoder              mov     encoderChange, zero
                        and     newscan, #%11                 
                        cmp     newscan, oldScan        wz      ' check for change
              if_z      jmp     #chkencoder_ret 'encloop 'chkencoder ' ' wait if none

                        'there is a change
                        
                        add     pulsePtr, pulseHead
                        mov     newPulse, cnt
                        rdlong  oldPulse, pulsePtr
                        sub     pulseTotal, oldPulse
                        wrlong  newPulse, lastUpdatedPtr
                     
                        mov     temp, newPulse ' store this loop's cnt value in lastPulse

                        sub     temp, lastPulse
                        mov     lastPulse, newPulse
                    
                        add     pulseTotal, temp
                        mov     tempTotal, pulseTotal
                        shr     tempTotal, #BITS_TO_AVERAGE_ENCODER_TIME
                        wrlong  tempTotal, pulseBufferSumPtr  
                        
                                     
                        add     pulseHead, #4
                        cmpsub  pulseHead, headRollover                
                        
                        wrlong  temp, pulsePtr             
                        
                        
case11                  cmp     oldscan, #%11           wz      ' oldscan == %11?         
              if_nz     jmp     #case01                         ' if no, try next         
                        cmp     newscan, #%01           wz      ' clockwise (postive)? Z = yes 
                        jmp     #update

case01                  cmp     oldscan, #%01           wz
              if_nz     jmp     #case00
                        cmp     newscan, #%00           wz   
                        jmp     #update                        

case00                  cmp     oldscan, #%00           wz      
              if_nz     jmp     #case10                         
                        cmp     newscan, #%10           wz      
                        jmp     #update                      
                                                                          
case10                  cmp     oldscan, #%10           wz
              if_nz     jmp     #chkencoder_ret                 ' (should never happen)
                        cmp     newscan, #%11           wz

update        if_z      mov     encoderChange, one                ' increment value

decvalue      if_nz     mov     encoderChange, negOne             
                        wrlong  encoderChange, directionFlagPtr
                        add     encoderTotal, encoderChange
writeTotal              wrlong  encoderTotal, encoderAddress
                                               
chkencoder_ret          ret

'-------------------------------------------------------------------------------

zero                    long 0
one                     long 1
negOne                  long -1
timeAddress             long 0
encoderAddress          long 0
encoderTotal            long 0 
encoderTotal0           long 0 
encoderTotal1           long 0
encoderChange           long 0
encoderQuanta0          long 0
encoderQuanta1          long 0
encoderPin0             long 0
encoderPin1             long 0

' first location in each buffer
pulsePtr                long 0
pulsePtr0               long 0
pulsePtr1               long 0

' current location in each buffer

pulseHead               long 0
pulseHead0              long 0
pulseHead1              long 0

tempHead                long 0
bufferTotal0            long 0
bufferTotal1            long 0
pulseTotal              long 0
pulseTotal0             long 0 
pulseTotal1             long 0

pulseBufferSumPtr       long 0
pulseBufferSumPtr0      long 0
pulseBufferSumPtr1      long 0
lastUpdatedPtr          long 0
lastUpdatedPtr0         long 0
lastUpdatedPtr1         long 0
headRollover            long 0

directionFlagPtr        long 0
directionFlagPtr0       long 0
directionFlagPtr1       long 0
encoderAddress0         res 1
encoderAddress1         res 1
oldScan                 res 1
oldscan0                res 1
oldscan1                res 1
originalScan            res 1
newscan                 res 1
difference              res 1
oldQuanta               res 1
oldPulse                res 1
temp                    res 1
lastPulse               res 1  
lastPulse0              res 1
lastPulse1              res 1
newPulse                res 1
tempTotal               res 1
previousTime            res 1
loopTimer               res 1
deltaTime               res 1

              fit
              
DAT '' License
{{
*********************************************************************************************
*                             TERMS OF USE: MIT License                                     *                                                                   
*********************************************************************************************
* Permission is hereby granted, free of charge, to any person obtaining a copy of this      *
* software and associated documentation files (the "Software"), to deal in the Software     *
* without restriction, including without limitation the rights to use, copy, modify,        *
* merge, publish, distribute, sublicense, and/or sell copies of the Software, and to        *
* permit persons to whom the Software is furnished to do so, subject to the following       *
* conditions:                                                                               *
*                                                                                           *                                    
* The above copyright notice and this permission notice shall be included in all copies or  *
* substantial portions of the Software.                                                     *
*                                                                                           *                                   
* The software is provided "as is", without warranty of any kind, express or implied,       *
* including but not limited to the warranties of merchantability, fitness for a particular  *
* and noninfringment. In no event shall the authors or copyright holders be liable for any  *
* claim, damages or other liability, whether in an action of contract, tort or otherwise,   *
* arising from, of or in connection with the software of the use of other dealings in the   *
* software.                                                                                 *
*********************************************************************************************
}}        